@@ -30,9 +30,12 @@ class CaptchaKindEnum(Enum):
 class MITMRecord:
     kind: CaptchaKindEnum
     sitekey: str
+    data_action: str
 
 
-__dir__ = path.dirname(path.abspath(__file__))
+# support for pyarmor/pyinstaller
+__dir__ = path.join(getattr(sys, '_MEIPASS'), 'harvester', 'server') if getattr(
+    sys, '_MEIPASS', None) else path.abspath(path.dirname(__file__))
 MITM_CAHCE: Dict[str, MITMRecord] = {}
 tokens: 'ExpiringQueue[str]' = ExpiringQueue(110)
 
@@ -89,10 +92,14 @@ class ProxyHTTPRequestHandler(BaseHTTPRequestHandler):
                 if token:
                     tokens.put(token)
             self._simple_headers(200, 'text/html; charset=utf-8')
-            self._render_template(self.config.kind.value + '.html',
-                                  domain=self.path,
-                                  sitekey=self.config.sitekey,
-                                  server=f"https://{host}:{port}")
+
+            kwargs = dict(domain=self.path, sitekey=self.config.sitekey,
+                          server=f"https://{host}:{port}")
+
+            if self.config.kind == CaptchaKindEnum.RECAPTCHA_V3:
+                kwargs['action'] = self.config.data_action
+
+            self._render_template(self.config.kind.value + '.html', **kwargs)
         elif self.path.endswith('.pac'):
             domain = self.path[1:-4]
             self._simple_headers(200, 'text/plain; charset=utf-8')
@@ -126,13 +133,15 @@ class ProxyHTTPRequestHandler(BaseHTTPRequestHandler):
         })
 
 
-def setup(server_address: Tuple[str, int], domain: str, captcha_kind: CaptchaKindEnum, sitekey: str) -> ThreadingHTTPServer:
-    MITM_CAHCE[domain] = MITMRecord(captcha_kind, sitekey)
+def setup(server_address: Tuple[str, int], domain: str, captcha_kind: CaptchaKindEnum, sitekey: str,
+          data_action: str = None, keyfile: str = None, certfile: str = None) -> ThreadingHTTPServer:
+    MITM_CAHCE[domain] = MITMRecord(captcha_kind, sitekey, data_action)
     httpd = ThreadingHTTPServer(server_address, ProxyHTTPRequestHandler)
     httpd.domain = domain
     httpd.socket = ssl.wrap_socket(httpd.socket,
-                                   keyfile=path.join(__dir__, 'server.key'),
-                                   certfile=path.join(__dir__, 'server.crt'), server_side=True)
+                                   keyfile=keyfile or path.join(
+                                       __dir__, 'server.key'),
+                                   certfile=certfile or path.join(__dir__, 'server.crt'), server_side=True)
     return httpd
 
 
@@ -145,6 +154,8 @@ def serve(httpd: ThreadingHTTPServer):
         httpd.shutdown()
 
 
-def start(server_address: Tuple[str, int], domain: str, captcha_kind: CaptchaKindEnum, sitekey: str):
-    httpd = setup(server_address, domain, captcha_kind, sitekey)
+def start(server_address: Tuple[str, int], domain: str, captcha_kind: CaptchaKindEnum, sitekey: str,
+          keyfile: str = None, certfile: str = None):
+    httpd = setup(server_address, domain, captcha_kind, sitekey, keyfile,
+                  certfile)
     serve(httpd)
